﻿using System;
using Callisto.Controls;
using Microsoft.WindowsAzure.MobileServices;
using wAzureAppWin8.ObjectModel;
using wAzureAppWin8.Pages;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Activation;
using Windows.Networking.PushNotifications;
using Windows.UI;
using Windows.UI.ApplicationSettings;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;

// The Blank Application template is documented at http://go.microsoft.com/fwlink/?LinkId=234227

namespace wAzureAppWin8
{
    /// <summary>
    /// Provides application-specific behavior to supplement the default Application class.
    /// </summary>
    sealed partial class App : Application
    {
        public static MobileServiceClient MobileService = new MobileServiceClient(
            "https://desmobiledemo.azure-mobile.net/",
            ".........");

        /// <summary>
        /// Initializes the singleton application object.  This is the first line of authored code
        /// executed, and as such is the logical equivalent of main() or WinMain().
        /// </summary>
        public App()
        {
            this.InitializeComponent();
            this.Suspending += OnSuspending;
        }

        /// <summary>
        /// Invoked when the application is launched normally by the end user.  Other entry points
        /// will be used when the application is launched to open a specific file, to display
        /// search results, and so forth.
        /// </summary>
        /// <param name="args">Details about the launch request and process.</param>
        protected override void OnLaunched(LaunchActivatedEventArgs args)
        {
            this.Initialize();

            Frame rootFrame = Window.Current.Content as Frame;

            // Do not repeat app initialization when the Window already has content,
            // just ensure that the window is active
            if (rootFrame == null)
            {
                // Create a Frame to act as the navigation context and navigate to the first page
                rootFrame = new Frame();

                if (args.PreviousExecutionState == ApplicationExecutionState.Terminated)
                {
                    //TODO: Load state from previously suspended application
                }

                // Place the frame in the current Window
                Window.Current.Content = rootFrame;
            }

            if (rootFrame.Content == null)
            {
                // When the navigation stack isn't restored navigate to the first page,
                // configuring the new page by passing required information as a navigation
                // parameter
                if (!rootFrame.Navigate(typeof(MainPage), args.Arguments))
                {
                    throw new Exception("Failed to create initial page");
                }
            }
            // Ensure the current window is active
            Window.Current.Activate();
        }

        private async void Initialize()
        {
            var a = AuthenticationHelper.Instance;
            SettingsPane.GetForCurrentView().CommandsRequested += App_CommandsRequested;

            var channel = await PushNotificationChannelManager.CreatePushNotificationChannelForApplicationAsync();

            var table = MobileService.GetTable<PushChannel>();

            await table.InsertAsync(new PushChannel() 
            {
                Data = channel.Uri,
                Type = "W8"
            });
        }

        private void App_CommandsRequested(SettingsPane sender, SettingsPaneCommandsRequestedEventArgs args)
        {
            args.Request.ApplicationCommands.Add(new SettingsCommand("account", "Account", OpenAccountSettings));
        }

        private void OpenAccountSettings(IUICommand command)
        {
            SettingsFlyout settings = new SettingsFlyout();

            settings.FlyoutWidth = SettingsFlyout.SettingsFlyoutWidth.Narrow;
            settings.HeaderText = command.Label;
            settings.Background = new SolidColorBrush(Colors.Black);
            settings.HeaderBrush = (SolidColorBrush)settings.Foreground;
            settings.ContentBackgroundBrush = (SolidColorBrush)settings.Background;
            settings.ContentForegroundBrush = (SolidColorBrush)settings.Foreground;
            settings.Content = new AccountSettings();
            settings.IsOpen = true;
        }

        /// <summary>
        /// Invoked when application execution is being suspended.  Application state is saved
        /// without knowing whether the application will be terminated or resumed with the contents
        /// of memory still intact.
        /// </summary>
        /// <param name="sender">The source of the suspend request.</param>
        /// <param name="e">Details about the suspend request.</param>
        private void OnSuspending(object sender, SuspendingEventArgs e)
        {
            var deferral = e.SuspendingOperation.GetDeferral();
            //TODO: Save application state and stop any background activity
            deferral.Complete();
        }
    }
}
