﻿using System.Linq;
using System.Threading.Tasks;
using Microsoft.WindowsAzure.MobileServices;
using Windows.Data.Json;
using Windows.Security.Credentials;
using Windows.Storage;

namespace wAzureAppWin8.ObjectModel
{
    public class AuthenticationHelper
    {
        private PasswordVault _vault = new PasswordVault();
        private PasswordCredential _credential;

        public bool IsAuthenticated
        {
            get { return App.MobileService.CurrentUser != null; }
        }

        public async Task LoginAsync()
        {
            var result = await App.MobileService.LoginAsync(MobileServiceAuthenticationProvider.Twitter);

            if (result.MobileServiceAuthenticationToken != null)
            {
                _credential = new PasswordCredential("token", result.UserId, result.MobileServiceAuthenticationToken);
                _vault.Add(_credential);
            }
        }

        public void Logout()
        {
            _vault.RetrieveAll();

            App.MobileService.Logout();
        }

        private AuthenticationHelper()
        {
            _credential = _vault.RetrieveAll().FirstOrDefault(x => x.Resource == "token");
            this.Initialize();
        }

        private void Initialize()
        {
            if (_credential != null)
            {
                _credential.RetrievePassword();
                App.MobileService.CurrentUser = new MobileServiceUser(_credential.UserName) 
                {
                    MobileServiceAuthenticationToken = _credential.Password
                };
            }
        }

        public static readonly AuthenticationHelper Instance = new AuthenticationHelper();
    }
}
