using System;
using System.Collections.ObjectModel;
using System.Windows.Input;
using Callisto.Controls;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using Microsoft.WindowsAzure.MobileServices;
using wAzureAppWin8.ObjectModel;
using wAzureAppWin8.Pages;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls.Primitives;

namespace wAzureAppWin8.ViewModel
{
    public class MainViewModel : ViewModelBase
    {
        private IMobileServiceTable<Message> _table;

        public MainViewModel()
        {
            _table = App.MobileService.GetTable<Message>();
            this.LoadData();
        }

        private ObservableCollection<Message> _messages;
        public ObservableCollection<Message> Messages
        {
            get { return _messages; }
            set { this.Set("Messages", ref _messages, value); }
        }

        private ICommand _refreshCommand;
        public ICommand RefreshCommand
        {
            get
            {
                if (_refreshCommand == null)
                {
                    _refreshCommand = new RelayCommand(() =>
                    {
                        this.LoadData();
                    });
                }

                return _refreshCommand;
            }
        }

        private async void LoadData()
        {
            try
            {
                var result = await _table.ToListAsync();

                this.Messages = new ObservableCollection<Message>(result);
            }
            catch (Exception)
            {
                new MessageDialog("Errore durante la richiesta").ShowAsync();
            }
        }

        private ICommand _openFlyOut;
        public ICommand OpenFlyOut
        {
            get
            {
                if (_openFlyOut == null)
                {
                    _openFlyOut = new RelayCommand<UIElement>(async x =>
                    {
                        //if (!AuthenticationHelper.Instance.IsAuthenticated)
                        //{
                        //    await new MessageDialog("Devi essere autenticato per inviare messaggi").ShowAsync();
                        //    return;
                        //}

                        Flyout flyout = new Flyout();
                        flyout.PlacementTarget = x;
                        flyout.Placement = PlacementMode.Top;
                        flyout.Content = new NewMessagePage(() => this.LoadData());

                        flyout.IsOpen = true;

                    });
                }

                return _openFlyOut;
            }
        }
    }
}