﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using Caliburn.Micro;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Notification;
using wAzureAppWp8.ObjectModel;
using wAzureAppWp8.ViewModels;

namespace wAzureAppWp8
{
    public class AppBootstrapper : PhoneBootstrapper
    {
        public static HttpNotificationChannel PushChannel { get; set; }
        public static PhoneContainer Container;

        protected override void Configure()
        {
            Container = new PhoneContainer(RootFrame);

            Container.RegisterPhoneServices();
            Container.PerRequest<MainPageViewModel>();
            Container.PerRequest<SettingsViewModel>();
            Container.Singleton<EventAggregator>();

            LogManager.GetLog = x => new Logger();
            
            AddCustomConventions();

            SetupPushNotification();
        }

        private async void SetupPushNotification()
        {
            try
            {
                PushChannel = HttpNotificationChannel.Find("DemoPush");

                if (PushChannel == null)
                {
                    PushChannel = new HttpNotificationChannel("DemoPush");
                    PushChannel.Open();
                    PushChannel.BindToShellToast();
                }

                PushChannel.ChannelUriUpdated += PushChannel_ChannelUriUpdated;
            }
            catch (Exception ex)
            {
                throw;
            }
            
        }

        private async void PushChannel_ChannelUriUpdated(object sender, NotificationChannelUriEventArgs e)
        {
            await App.MobileService.GetTable<PushChannel>().InsertAsync(new PushChannel()
            {
                Type = "WP8",
                Data = PushChannel.ChannelUri.ToString()
            });
        }

        protected override object GetInstance(Type service, string key)
        {
            return Container.GetInstance(service, key);
        }

        protected override IEnumerable<object> GetAllInstances(Type service)
        {
            return Container.GetAllInstances(service);
        }

        protected override void BuildUp(object instance)
        {
            Container.BuildUp(instance);
        }

        static void AddCustomConventions()
        {
            ConventionManager.AddElementConvention<Pivot>(Pivot.ItemsSourceProperty, "SelectedItem", "SelectionChanged").ApplyBinding =
                (viewModelType, path, property, element, convention) =>
                {
                    if (ConventionManager
                        .GetElementConvention(typeof(ItemsControl))
                        .ApplyBinding(viewModelType, path, property, element, convention))
                    {
                        ConventionManager
                            .ConfigureSelectedItem(element, Pivot.SelectedItemProperty, viewModelType, path);
                        ConventionManager
                            .ApplyHeaderTemplate(element, Pivot.HeaderTemplateProperty, null, viewModelType);
                        return true;
                    }

                    return false;
                };

            ConventionManager.AddElementConvention<Panorama>(Panorama.ItemsSourceProperty, "SelectedItem", "SelectionChanged").ApplyBinding =
                (viewModelType, path, property, element, convention) =>
                {
                    if (ConventionManager
                        .GetElementConvention(typeof(ItemsControl))
                        .ApplyBinding(viewModelType, path, property, element, convention))
                    {
                        ConventionManager
                            .ConfigureSelectedItem(element, Panorama.SelectedItemProperty, viewModelType, path);
                        ConventionManager
                            .ApplyHeaderTemplate(element, Panorama.HeaderTemplateProperty, null, viewModelType);
                        return true;
                    }

                    return false;
                };
        }

#if DEBUG
        protected override void OnUnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            base.OnUnhandledException(sender, e);

            MessageBox.Show(e.ExceptionObject.ToString());
        }
#endif
        
    }
}
