using System;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Input;
using Caliburn.Micro;
using Microsoft.WindowsAzure.MobileServices;
using wAzureAppWp8.ObjectModel;

namespace wAzureAppWp8.ViewModels
{
    public class MainPageViewModel : PropertyChangedBase
    {
        private IMobileServiceTable<ChatMessage> _table;
        private INavigationService _navigationService;

        public MainPageViewModel(INavigationService navigationService)
        {
            _table = App.MobileService.GetTable<ChatMessage>();
            _navigationService = navigationService;
            this.LoadData();
        }

        private ObservableCollection<ChatMessage> _messages;
        public ObservableCollection<ChatMessage> Messages
        {
            get { return _messages; }
            set 
            {
                _messages = value;
                this.NotifyOfPropertyChange(() => this.Messages);
            }
        }

        public async void LoadData()
        {
            try
            {
                var result = await _table.ToListAsync();

                this.Messages = new ObservableCollection<ChatMessage>(result);
            }
            catch (Exception)
            {
                MessageBox.Show("Errore durante la richiesta");
            }
        }

        public void GoToSettings()
        {
            _navigationService.UriFor<SettingsViewModel>().Navigate();
        }

        private string _newMessage;
        public string NewMessage
        {
            get { return _newMessage; }
            set
            {
                _newMessage = value;
                this.NotifyOfPropertyChange(() => this.NewMessage);
            }
        }

        public async void Send()
        {
            try
            {
                if (!AuthenticationHelper.Instance.IsAuthenticated)
                {
                    MessageBox.Show("Devi prima autenticarti!");
                    return;
                }

                if (string.IsNullOrWhiteSpace(this.NewMessage))
                    return;

                var message = new ChatMessage()
                {
                    Body = this.NewMessage
                };

                await _table.InsertAsync(message);

                this.NewMessage = string.Empty;
                this.LoadData();
            }
            catch (Exception)
            {
                MessageBox.Show("Errore durante la richiesta");
            }
        }
    }
}