//
//  MainViewController.m
//  wazureapp
//
//  Created by Marco De Sanctis on 30/12/12.
//  Copyright (c) 2012 Marco De Sanctis. All rights reserved.
//

#import "MainViewController.h"

#import "MSClient_Interface.h"
#import <WindowsAzureMobileServices/WindowsAzureMobileServices.h>
#import "MessageCellView.h"

#define NotifyKeyboard() \
NSNotificationCenter *notificationCenter = [NSNotificationCenter defaultCenter]; \
[notificationCenter addObserver:self selector:@selector(keyboardWillShow:) name:UIKeyboardWillShowNotification object:nil]; \
[notificationCenter addObserver:self selector:@selector(keyboardWillShow:) name:UIKeyboardWillHideNotification object:nil]

@interface MainViewController ()

@property (nonatomic, retain) NSArray *items;

@end

@implementation MainViewController

@synthesize items;
@synthesize theTable;
@synthesize txtNewMessage;
@synthesize sendButton;

MSTable* _table = nil;

#pragma keyboard
float original = 0;

UIKIT_STATIC_INLINE void UIViewSetFrameY(UIView *view, CGFloat y, bool closing) {
    NSLog(@"from %f to %f", view.frame.origin.y, y);
    if (!closing)
        original = view.frame.origin.y;
    else
        y = original;
    
    view.frame = CGRectMake(view.frame.origin.x, y, view.frame.size.width, view.frame.size.height);
}

- (void)keyboardWillShow:(NSNotification *)notification {
    NSTimeInterval animationDuration;
    UIViewAnimationCurve animationCurve;
    CGRect frameEnd;
    NSDictionary *userInfo = [notification userInfo];
    [[userInfo objectForKey:UIKeyboardAnimationCurveUserInfoKey] getValue:&animationCurve];
    [[userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey] getValue:&animationDuration];
    [[userInfo objectForKey:UIKeyboardFrameEndUserInfoKey] getValue:&frameEnd];
    
    bool showing = notification.name == UIKeyboardWillShowNotification;
    
    //    NSLog(@"animationDuration: %f", animationDuration); // TODO: Why 0.35 on viewDidLoad?
    [UIView animateWithDuration:animationDuration delay:0.0 options:UIViewAnimationOptionBeginFromCurrentState animations:^{
        CGFloat viewHeight = [self.view convertRect:frameEnd fromView:nil].origin.y;
        UIView *messageInputBar = self.txtNewMessage.superview;
        UIViewSetFrameY(messageInputBar, viewHeight-messageInputBar.frame.size.height, !showing);
        self.theTable.contentInset = self.theTable.scrollIndicatorInsets = UIEdgeInsetsMake(0, 0, self.view.frame.size.height-viewHeight, 0);
        [self scrollToBottomAnimated:NO];
    } completion:nil];
}

- (void)scrollToBottomAnimated:(BOOL)animated {
    NSInteger numberOfRows = [self.theTable numberOfRowsInSection:0];
    if (numberOfRows) {
        [self.theTable scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:numberOfRows-1 inSection:0] atScrollPosition:UITableViewScrollPositionBottom animated:animated];
    }
}

- (void)hideKeyboard:(id)sender
{
    [self.txtNewMessage resignFirstResponder];
}

#pragma load data and init

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        self.tabBarItem = [[[UITabBarItem alloc] initWithTitle:@"Chat" image:nil tag:0] autorelease];
    }
    return self;
}

- (void)RefreshData:(id)sender {
    [_table readWhere:nil completion:^(NSArray *readItems, NSInteger totalCount, NSError *error) {
        self.items = [readItems mutableCopy];
        
        [self.theTable reloadData];
        
        if (self.items.count > 0) {
//            [theTable scrollToRowAtIndexPath:[NSIndexPath indexPathForItem:self.items.count - 1 inSection:1] atScrollPosition:UITableViewScrollPositionBottom animated:true];
        }
    }];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(hideKeyboard:)];
    tap.cancelsTouchesInView = FALSE;
    [self.theTable addGestureRecognizer:tap];
    [tap release];
    
    
    _table = [[[MSClient getInstance] getTable:@"Messages"] retain];
    
    [self RefreshData:self];
    
    NotifyKeyboard();
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)dealloc {
    [_table release];
    [theTable release];
    [items release];
    [txtNewMessage release];
    [sendButton release];
    
    [super dealloc];
}

- (IBAction)Click:(id)sender {
    UIAlertView *view = [[UIAlertView alloc] initWithTitle:@"test" message:@"test" delegate:self cancelButtonTitle:@"ok" otherButtonTitles:nil, nil];
    
    [view show];
    
    [view release];
}

#pragma TableViewDataSource
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {

    if (self.items == nil)
        return 0;
    
    return self.items.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView
         cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    MessageCellView *theCell = [tableView dequeueReusableCellWithIdentifier:@"theCell"];
    
    if (theCell == nil) {
        NSArray *array = [[NSBundle mainBundle] loadNibNamed:@"MessageCellView" owner:nil options:nil];
        
        theCell = [array objectAtIndex:0];
    }
    
    NSDictionary *item = [self.items objectAtIndex:indexPath.row];
    theCell.body.text = [item objectForKey:@"body"];
    
    if ((NSNull *) [item objectForKey:@"sender"] != [NSNull null]) {
        theCell.sender.text = [item objectForKey:@"sender"];
    }
    [theCell setImageUrl: [item objectForKey:@"pictureUrl"]];
    
    return theCell;
    
}
- (void)viewDidUnload {
    [self setTheTable:nil];
    [self setTxtNewMessage:nil];
    [super viewDidUnload];
}
- (IBAction)SendMessage:(id)sender {
    if ([self.txtNewMessage.text length] == 0)
        return;
    
    NSMutableDictionary  *newItem = [[NSMutableDictionary alloc] init];
    [newItem setObject:self.txtNewMessage.text forKey:@"body"];
    
    [_table insert:newItem completion:^(NSDictionary *item, NSError *error) {
        if (error != nil) {
            NSLog(@"%@", error);
            
            NSString* message = error.code == -1302 ? @"Bisogna autenticarsi prima di postare" :
            @"Si è verificato un errore durante l'invio del dato";
            
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Errore" message:message delegate:self cancelButtonTitle:@"Ok" otherButtonTitles:nil, nil];
            
            [alert show];
            
            [alert release];
            
        } else {
            [self RefreshData:self];
        }
    }];
    
    [newItem release];
    
    [self.txtNewMessage resignFirstResponder];
}

- (void)textViewDidChange:(UITextView *)textView {
    self.sendButton.enabled = [self.txtNewMessage.text length] > 0;
}
@end
